function [F, J] = i_trick(fun, x, param)

% Calculates the Jacobian of function "fun" in "x" using the imaginary trick
% F: the function value in "x"
% J: Jacobian of "fun" in "x"

% Definining the step length
h  = 1e-100;

% Evaluate once to get dimension
F  = fun(x, param);

nz = length(x);

% Initialize Jacobian
J  = zeros(length(F), nz);

% In each loop we obtain one column of the Jacobian (p is the seed)
p = zeros(nz, 1);
for k = 1 : nz
    p(k) = 1i * h;
    J(:, k) = imag( fun(x + p,param)/h );
    p(k) = 0;
end

end