% simple Newton method
clear; close all; clc

%% initial definitions

% define y = [x1; x2; lambda]
% first order necessary conditions: F(y) = 0
F = @(y)([-2 * y(3) * y(1); 1 - 2 * y(3) * y(2); y(1)^2 + y(2)^2 - 1]);

% Jacobian of root finding problem <=> KKT matrix
% We're using exact Newton's method
J = @(y)([-2 * y(3), 0, -2 * y(1);
          0, -2 * y(3), -2 * y(2);
          2 * y(1), 2 * y(2), 0]);

% random initialization of y
% distribituted uniformly random in box around origin
y = 2 * (rand(3,1) -.5 ) ;

tol = 1e-5;                 % convergence tolerance
maxit = 30;                 % maximum iterations

%% main loop
for i = 1:maxit
    % check convergence
    if max(abs(F(y))) < tol
        disp(['solution found in ' num2str(i-1) ' iterations'])
        break
    end
    % take full Newton step
    y = y - J(y)\F(y);
end

if i >= maxit
    disp('maximum iterations reached')
end

disp(' ')
disp(y)